package com.wordpress.salaboy.examples;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.base.MapGlobalResolver;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.impl.ClassPathResource;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;
import org.drools.runtime.process.WorkflowProcessInstance;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * Created by IntelliJ IDEA. User: salaboy Date: 2/14/11 Time: 9:58 AM To change
 * this template use File | Settings | File Templates.
 */
public class SimpleEmergencyProcessTest1 {
	private static StatefulKnowledgeSession ksession;

	public SimpleEmergencyProcessTest1() {
	}
	
	   /**
     * We are using JTA so we need a JTA-enabled DataSource
     */
    private static PoolingDataSource ds1;
    
    
    /**
     * Data source creation
     */
    @BeforeClass
    public static void createDataSource() {
        ds1 = new PoolingDataSource();
        ds1.setUniqueName("jdbc/processInstanceDS");
        ds1.setClassName("com.mysql.jdbc.jdbc2.optional.MysqlXADataSource");
        ds1.setMaxPoolSize(3);
        ds1.setAllowLocalTransactions(true);
//        ds1.getDriverProperties().setProperty("driverClassName", "com.mysql.jdbc.Driver");
        ds1.getDriverProperties().put("user", "root");
        ds1.getDriverProperties().put("password", "nagarro1136");
        ds1.getDriverProperties().put("url", "jdbc:mysql://localhost/jbpmTest?user=root&password=nagarro1136");
        ds1.init();
    }
    
    /**
     * Data source cleaning
     */
    @AfterClass
    public static void closeDataSource() {
        if (ds1 != null){
            ds1.close();
        }
    }

	/*
	 * This test shows how to interact with a business process that contains
	 * human tasks activities using an Automatic Human Task Activities
	 * simulator. This business process also contains a BusinessRuleTask
	 * activity defined. A set of rules will be evaluated when this activity is
	 * executed by the process. Take a look at the comments in the test for more
	 * explanations. When a Human Task is found inside the business process, the
	 * MyHumanActivitySimulatorWorkItemHandler automatically completes the task
	 * causing the process to advance to the next activity.
	 */
	@Test
	public void automaticHumanTasksTest() throws InterruptedException {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(new ClassPathResource("EmergencyServiceSimple.bpmn"),
				ResourceType.BPMN2);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error : errors) {
				System.out.println(error.getMessage());

			}
			return;
		}

		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.jbpm.task");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
//		 env.set(EnvironmentName.TRANSACTION_MANAGER,
//			        TransactionManagerServices.getTransactionManager());
//		
//		 env.set(EnvironmentName.GLOBALS, new MapGlobalResolver());

//         env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());

//		Properties properties = new Properties();
//		properties
//				.put("drools.processInstanceManagerFactory",
//						"org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
//		properties.put("drools.processSignalManagerFactory",
//				"org.jbpm.persistence.processinstance.JPASignalManagerFactory");
//		KnowledgeSessionConfiguration config = KnowledgeBaseFactory
//				.newKnowledgeSessionConfiguration(properties);

		final StatefulKnowledgeSession ksession = JPAKnowledgeService
				.newStatefulKnowledgeSession(kbase, null, env);

//		ksession = kbase.newStatefulKnowledgeSession();
		MyAutomaticHumanSimulatorWorkItemHandler humanActivitiesSimHandler = new MyAutomaticHumanSimulatorWorkItemHandler();
		ksession.getWorkItemManager().registerWorkItemHandler("Human Task",
				humanActivitiesSimHandler);

//		JPAWorkingMemoryDbLogger logger = new JPAWorkingMemoryDbLogger(ksession);
		
//		KnowledgeRuntimeLoggerFactory.newConsoleLogger(ksession);
		
	

//		Thread t = new Thread(new Runnable() {
//			
//			@Override
//			public void run() {
//				ksession.fireUntilHalt();
//			}
//		});
//		
//		t.setDaemon(true);
//		t.start();
		
		WorkflowProcessInstance process = (WorkflowProcessInstance) ksession
				.startProcess("com.wordpress.salaboy.bpmn2.SimpleEmergencyService");
		
		

		// I need to call the FireAllRules method because I have a RuleTask in
		// my business process
		

		ksession.fireAllRules();
		
		// Is the process completed?
		Assert.assertEquals(ProcessInstance.STATE_COMPLETED, process.getState());
		

	}

	class MyAutomaticHumanSimulatorWorkItemHandler implements WorkItemHandler {

		public void executeWorkItem(WorkItem workItem,
				WorkItemManager workItemManager) {
//			System.out.println("Map of Parameters = "
//					+ workItem.getParameters());
			workItemManager.completeWorkItem(workItem.getId(), null);
		}

		public void abortWorkItem(WorkItem workItem,
				WorkItemManager workItemManager) {

		}

	}
}
